function [left_bndry,right_bndry] = fb_StblBound(Me,Ke,C,tau)
% [left_bndry,right_bndry] = fb_StblBound(Me,Ke,C,tau)
%
% Computes the stability boundary for "kp" and "kv" for the
% system of the form:
%      Me*qA''(t) - Ke*qA(t) + C*[kp*qA(t-tau)+kv*qA'(t-tau)] = 0
%
% The outputs are of the form:
%        left_bndry  = [kp,kv]
%        right_bndry = [kp,kv]
%
%::: Updated 2012.02.15 :::

% Tolerances for finding peak values
tol_amax = 1e-8;
tol_itermax = 1e3;

% Determine the left-hand boundary
%  kp = Ke/C
%  kv = all values
Kp_min = Ke/C;
Kv_min = Ke*tau/C;


% Test if the delay is stable
T_Max = sqrt(2*Me/Ke);
if (tau>T_Max)
    warning('fb_ndStblBound:DelayUnstable','System is unstable. Delay > %f',T_Max);
    left_bndry = [Kp_min,Kv_min];
    right_bndry = [Kp_min,Kv_min];
    return;
end

% Calculate the right-hand boundary using parameterized frequency, w
% Kp = cos(tau*w).*(Me*w.^2+Ke)/C;
% Kv = sin(tau*w).*(Me*w.^2+Ke)/C./w;

% Find the upper left-hand limit of the boundary
w_end = pi/2/tau;
w = linspace(eps,w_end,1000);
Kp = cos(tau*w).*(Me*w.^2+Ke)/C;
[~,idx] = max(Kp);
[w_end,fval,EXITFLAG] = fminbnd(@(w) abs(cos(tau*w).*(Me*w.^2+Ke)/C-Kp_min),w(idx),w(end),optimset('TolX',1e-12,'TolFun',tol_amax,'MaxIter',tol_itermax,'Display','off'));
if (EXITFLAG~=1)
    warning('fb_ndStblBound:BoundaryLimitsBad','Upper-left hand limits only calculated to tolerance of: %0.4g.',fval);
end

w = linspace(eps,w_end,1000);
Kp = cos(tau*w).*(Me*w.^2+Ke)/C;
Kv = sin(tau*w).*(Me*w.^2+Ke)/C./w;

left_bndry = [linspace(Kp_min,Kp_min)',linspace(min(Kv),max(Kv))'];
right_bndry = [Kp',Kv'];