function [Me, Ke, Ch, Cc] = fb_linCoeffs(humanHeight,humanMass,stanceRatio)
% [Me, Ke, Ch, Cc] = fb_linCoeffs(humanHeight,humanMass,stanceRatio)
%
% Computes the linearized coefficients of the fourbar linkage equations of
% motion.  Effective mass (Me) and effective gravitational stiffness (Ke)
% can then be used in the linearized inverted pendulum equation:
%        Me*qA'' = Ke*qA
% where qA is the rotational degree of freedom: ankle of the fourbar to
% ankle of inverted pendulum.
%
% C is the geometric transformation gain from hip to ankle
%       qA = Ch*qH = Cc*xCOM
%       Ch = (SW/W)*(SW/W)          % hip fdbk
%       Cc = (SW/W)*( ... )         % COM fdbk
%              (A)    (B)
%           A) This takes a torque applied at the hip and turns it into an
%              ankle torque.
%           B) This takes a displacement and turns it into an ankle
%              displacement.
%
%::: Updated 2012.01.23 :::

% Variables
G           =  9.81;          % m/s^2     Gravitational acceleration
HH          =  humanHeight;   % m         Height of human
HM          =  humanMass;     % kg        Mass of human
HCOM        =  0.108*HH;      % m         Height to HAT CoM from trochanter
L           =  0.530*HH;      % m         Leg length
LCOM        =  0.293*HH;      % m         Length to leg CoM from ground
MA          =  0.161*HM;      % kg        Mass of leg A
MB          =  0.678*HM;      % kg        Mass of trunk
MC          =  MA;            % kg        Mass of leg C
IA          =  0.0299*MA*HH^2;% kg*m^2    Moment of inertia for leg A
IB          =  0.0204*MB*HH^2;% kg*m^2    Moment of inertia for trunk
IC          =  IA;            % kg*m^2    Moment of inertia for leg C
W           =  0.191*HH*0.7;      % m         Hip width
SW          =  stanceRatio*W; % m         Stance width

% Linearized coefficients of fourbar equations of motion
% Me*qA'' = Ke*qA 
d = SW-W;
h = sqrt(4*L^2-d^2)/2;

% Horizontal CoM location
xcom = SW/2;
% Vertical CoM location
sinqa = h/L;
ycom = (2*LCOM*sinqa*MA+(L*sinqa+HCOM)*MB)/(HM);

Ke =-G*( (2*LCOM*MA+L*MB)*(d*h^2-L^2*SW)/L/W/h - (HCOM*d^2*MB)/W^2 );
Me = 2*(LCOM^2*MA+IA) + ( (HCOM*d-W*h)^2*MB + d^2*IB )/W^2;

% Configuration dependent gain for feedback if controlled variables are hip
% state
Ch  = (SW/W)^2;
% Configuration dependent gain for feedback if controlled variables are com
% horizontal displacement and velocity
Cc = (SW/W)*(W*(L*MB + 2*LCOM*MA)*(4*L^2 - (SW - W)^2)^(1/2) ...
    - 2*HCOM*L*MB*(SW - W))/(L*W*(4*MA + 2*MB));