clear,clc
% Master script for running the four-bar linkage model. It sets up and runs 
% solveFloppyEuler.m, fb_linCoeffs.m, and fb_StblBound.m

%%%%%%%%%%%% PARAMETERS %%%%%%%%%%%%%%
%----------+-----------------------+-----------+---------------------------
% Quantity | Value                 | Units     | Description
%----------|-----------------------|-----------|---------------------------

humanHeight = 1.8;                  % m         % nominal height of human
humanMass   = 70;                   % kg        % nominal mass of human
hipWidth    = 0.134*humanHeight;    % m         % width between hip joints
stanceRatio = 1.0;                  % m/m       % stanceWidth/hipWidth
stanceWidth = hipWidth*stanceRatio; % m         % width between feet
delay       = 0.1;                  % s         % feedback delay
Kf          = 0;                    % N*m/rad   % feedforward prop. gain
Kvf         = 0;                    % N*m*s/rad % feedforward vel. gain
Kb          = 600;                 % varies    % feedback prop. gain
Kvb         = Kb/3;                 % varies    % feedback vel. gain
cntrl       = 3;                    % N/A       % 2 for hip control (both joints)
pert_mag    = 0.4;                  % g         % standard perturbation mag. is 0.4g
SOLTIME     = [0 15];               % s         % start and stop times for simulation
addedMass   = 0;                    % kg        % add mass to model

%%%%%%%% RUN MODEL AND STABILITY BOUNDARIES %%%%%%%%%%%
output = solveFloppyEuler(humanHeight,humanMass,hipWidth,stanceWidth,delay,Kf,Kvf,Kb,Kvb,cntrl,pert_mag,SOLTIME,addedMass);
[Me, Ke, Ch, Cc] = fb_linCoeffs(humanHeight,humanMass,stanceRatio);
[output.stability.left_bndry,output.stability.right_bndry] = fb_StblBound(Me,Ke,Ch,delay);


figure()
hold on

%%% Plot model response %%%
subplot(2,6,1:6);
set(gcf,'Units','normalized',...
    'Position',[0.1 0.1 0.8 0.8])
set(gca,'FontSize',13)
line(output.solution.t,output.solution.com*100,...
    'LineWidth',2)
ylabel('COM Horizontal Position (cm)'),xlabel('time (s)')
title('Four-bar model response')

%%% Plot stability boundaries %%%
subplot(2,6,8:11);
set(gca,'FontSize',13)
line(output.stability.left_bndry(:,1),output.stability.left_bndry(:,2),...
    'Color',[0.5,0.7,0.5],...
    'LineWidth',2)
line(output.stability.right_bndry(:,1),output.stability.right_bndry(:,2),....
    'Color',[0.5,0.5,0.7],...
    'LineWidth',2)
line(Kb,Kvb,...
    'Color',[1,0,0],...
    'Marker','.',...
    'MarkerSize',10,...
    'LineStyle','none')
title('Stability Boundaries'),xlabel('K_b'),ylabel('K_v_b')

legend('Left Stability Boundary','Right Stability Boundary','User-selected Gains',...
    'Location','SouthEast')

clearvars -except output