% Updated 2013.02.26
%
% anOutput = solveFloppyEuler(humanHeight,humanMass,stanceRatio, delay,
%                             Kf,Kvf, Kb,Kvb, cntrl,pert_mag)
%
% ARGUMENTS:
%   humanHeight ........... (m) - Total height of person
%   humanMass ............ (kg) - Total mass of person
%   stanceRatio ......... (m/m) - Stance ratio (stance width / hip width)
%   delay ................  (s) - Feedback delay
%   Kf .............. (N*m/rad) - Feedforward stiffness gain
%   Kvf ........... (N*m*s/rad) - Feedforward damping gain
%   Kb ............... (varies) - Feedback stiffness gain
%   Kvb .............. (varies) - Feedback damping gain
%   cntrl ..................... - Switch to set control type:
%                                  1 - CoM feedback, torque on both hips
%                                  2 - Hip feedback, torque on both hips
%                                  3 - Hip feedback, torque on left hip
%                                  4 - CoM feedback, torque on left hip
%   pert_mag .............. (g) - Perturbation specified as an inertial 
%                                 acceleration can be specified in three
%                                 ways based on size of variable:
%                                 SIZE     DESCRIPTION
%                                 1 x 1    Specifies max acceleration and
%                                          assumes acceleration is two
%                                          gaussian pulses of opposite
%                                          sign.
%                                 n x 1    First column is acceleration and
%                                          it is equally spaced with an
%                                          assumed time vector equal to
%                                          simulation time.  Since this is
%                                          a lookup method this is fast.
%                                 n x 2    First column is time, second is
%                                          acceleration and interpolation
%                                          is used to calculate current
%                                          simulation value. This is slow.
%
% OUTPUTS:
%   anOutput ............... Structure containing the model parameters and
%   |                        solution vector.
%   |--gains ............... Structure holding the gains for the simulation
%   |--parameters .......... Structure holding body parameters for the sim
%   |--simulationStatus .... Tells how the simulation exited:
%   |                            0 - OK
%   |                            1 - Foot lift-off detected
%   |                            2 - Simulation terminated early
%   |--stancewidth ......... The stancewidth of the subject
%   |--solution ............ A matrix [NumSteps,22] holding the simulated
%                            model states.  Some important states are:
%   |--solution ............ A matrix [NumSteps,22] holding the simulated
%                            model states.  Some important states are:
%      solution.t .......... t - Simulation time
%      solution.q(:,1) ..... qA - ankle angle
%      solution.q(:,2) ..... qB - left hip angle
%      solution.q(:,3) ..... qC - right hip angle
%      solution.q(:,4) ..... qAp - ankle angular velocity
%      solution.q(:,5) ..... qBp - left hip angular velocity
%      solution.q(:,6) ..... qCp - right hip angular velocity
%      solution.grfx ....... FC1 - horizontal GRF under left foot
%      solution.grfy ....... FC2 - vertical GRF under left foot
%      solution.com ........ COM_X - center-of-mass horizontal displacement
%      solution.comv ....... COM_VX - CoM horizontal velocity
%      solution.TAB ........ TAB - left hip torque
%      solution.TBC ........ TBC - right hip torque
%
% SUPPORTING FUNCTIONS: NONE
%
% #########################################################################
function anOutput = solveFloppyEuler(humanHeight,humanMass,hipWidth,stanceWidth, delay, Kf,Kvf, Kb,Kvb, cntrl,pert_mag, SOLTIME, addedMass)
%
% Anthropometric values are taken from p60-65 of "Biomechanics and motor
% control of human movement" by DA Winter.  A website with a lot more
% anthropomorphic data is: http://www.dh.aist.go.jp/bodyDB

%----------+----------------+-----------+----------------------------------
% Quantity | Value          | Units     | Description
%----------|----------------|-----------|----------------------------------
G           =  9.81;         % m/s^2     Gravitational acceleration
HH          =  humanHeight;  % m         Height of human
HM          =  humanMass;    % kg        Mass of human
HCOM        =  0.108*HH;     % m         Height to HAT CoM from trochanter
L           =  0.530*HH;     % m         Leg length
LCOM        =  0.293*HH;     % m         Length to leg CoM from ground
MA          =  0.161*HM;     % kg        Mass of leg A
MB          =  0.678*HM;     % kg        Mass of trunk
MC          =  MA;           % kg        Mass of leg C
IA          =  0.0299*MA*HH^2; % kg*m^2    Moment of inertia for leg A
IB          =  0.0204*MB*HH^2+addedMass;  % kg*m^2    Moment of inertia for trunk
IC          =  IA;           % kg*m^2    Moment of inertia for leg C
W           =  hipWidth;%0.191*HH*0.7; % m         Hip width<--Narrow by 30% for HJC
SW          =  stanceWidth;%stanceRatio*W;% m         Stance width
TAB         =  0.0;          % N*m       Initial torque on hip AB
TBC         =  0.0;          % N*m       Initial torque on hip BC

qA          =  0;            % rad       Initial Value - requires solving
qB          =  0;            % rad       Initial Value - requires solving
qC          =  0;            % rad       Initial Value - requires solving
WCHECK1     =  0.0;          % UNITS     Initial Value - error checking
qAp         =  0.0;          % rad/s     Initial Value - start from rest

TINITIAL    =  SOLTIME(1);          % sec       Initial Time
TFINAL      =  SOLTIME(end);          % sec       Final Time
INTEGSTP    =  0.001;        % sec       Integration Step
ABSERR      =  1.0E-07;      %           Absolute Error
RELERR      =  1.0E-07;      %           Relative Error
%----------+----------------+-----------+----------------------------------

%<<<Initialize configuration>>>
% First determine initial ankle angle
qAo = acos((SW-W)/L/2); % SW = 2*L*cos(angleA) + W
% Now set the dependent angles to a symmetric configuration
qBo = -qAo;
qCo = -qAo;

% Set feedforward gains
FF_GAINS(1) = Kf; %Stiffness (0.05 Nm/rad from Jeven's thesis p32)
FF_GAINS(2) = Kvf; %Damping (0.005 Nms/rad from Jeven's thesis p32)
% Set feedback gains
FB_GAINS(1) = Kb; %Stiffness (0.4-0.6 Nm/rad from Jeven's thesis p32)
FB_GAINS(2) = Kvb; %Damping (0.04-0.06 Nms/rad from Jeven's thesis p32)
% Set feedback delay
FB_DELAY = floor(delay/INTEGSTP); %30 ms from Jeven's thesis

% Reserve space and initialize matrices
NUMSTEPS = ceil((TFINAL-TINITIAL)/INTEGSTP);
z = zeros(126,1);
aSolution = zeros(NUMSTEPS,22);

% Evaluate constants
z(12) = L - LCOM;
z(71) = G*MA;
z(73) = G*MB;
z(75) = G*MC;
z(80) = HCOM*z(73);
z(81) = W*z(73);
z(84) = IA + IB + IC + MA*LCOM^2;
z(86) = IB + IC;
z(88) = MC*z(12);
z(91) = IB + IC + 0.25*MB*(W^2+4*HCOM^2);
z(95) = IC + MC*z(12)^2;
z(112) = MA + MB + MC;
z(113) = (L*MB+LCOM*MA)/z(112);
z(114) = MB*W/z(112);
z(115) = HCOM*MB/z(112);
z(116) = LCOM*MC/z(112);
z(117) = MC*SW/z(112);
z(125) = IA + MA*LCOM^2;

% Set the initial values of the states
VAR(1) = qAo;
VAR(2) = qBo;
VAR(3) = qCo;
VAR(4) = WCHECK1; % energy
VAR(5) = qAp;     % velocity of ankle angle, e.g. qdot  


sysStatus = 0;
%<<<Begin numerical integration of EoM>>>
mdlDerivatives(0,VAR,1); %Initialized  u=1 is a flag to evaluate outputs
for sysTime=1:NUMSTEPS-1
    VAR = eulerStep(sysTime,VAR);
    if(mdlOutput(sysTime+1)~=0)             % mdlOutput flags errors/instablities in solution
        sysStatus = mdlOutput(sysTime+1);
    end
    if sysStatus==2
        break;
    end
end

%<<<Package data into a structure>>>
aSolution = downsample(aSolution,10);
anOutput.solution.t = aSolution(:,1);
anOutput.solution.q = aSolution(:,2:7);
anOutput.solution.com = aSolution(:,13);
anOutput.solution.comv = aSolution(:,15);
anOutput.solution.grfx = aSolution(:,11);
anOutput.solution.grfy = aSolution(:,12);
anOutput.solution.TAB = aSolution(:,18);
anOutput.solution.TBC = aSolution(:,19);

anOutput.parameters.HumanHeight = HH;
anOutput.parameters.HumanMass = HM;
anOutput.parameters.LegLength = L;
anOutput.parameters.LegCoM = LCOM;
anOutput.parameters.LegMass = MA;
anOutput.parameters.LegInertia= IA;
anOutput.parameters.TrunkCoM = HCOM;
anOutput.parameters.TrunkMass = MB;
anOutput.parameters.TrunkInertia = IB;
anOutput.parameters.HipWidth = W;
anOutput.gains.feedback.kp = FB_GAINS(1);
anOutput.gains.feedback.kv = FB_GAINS(2);
anOutput.gains.feedback.delay = FB_DELAY*INTEGSTP;
anOutput.gains.feedforward.kp = FF_GAINS(1);
anOutput.gains.feedforward.kv = FF_GAINS(2);
anOutput.stancewidth = SW;
anOutput.simulationStatus = sysStatus;
%==========================================================================
% eulerStep: Integrate forward one euler step
%==========================================================================
    function VARp = eulerStep(T,VAR)
        VARp = mdlDerivatives(T,VAR,1);
        for i=1:length(VAR)
            VARp(i) = VAR(i) + VARp(i)*INTEGSTP/2;
        end
        VARp = mdlDerivatives(T,VARp,0);
        for i=1:length(VAR)
            VARp(i) = VAR(i) + VARp(i)*INTEGSTP;
        end
    end

%==========================================================================
% mdlDerivatives: Calculates and returns the
%                 derivatives of the continuous states
%==========================================================================
    function sys = mdlDerivatives(T,VAR,u)
        % Update variables after integration step
        qA = VAR(1);
        qB = VAR(2);
        qC = VAR(3);
        WCHECK1 = VAR(4);
        qAp = VAR(5);
        z(5) = cos(qC);
        z(4) = sin(qB);
        z(7) = L*z(4);
        z(6) = sin(qC);
        z(3) = cos(qB);
        z(8) = L*z(3);
        z(11) = W + z(8);
        z(13) = z(5)*z(7) + z(6)*z(11);
        z(1) = cos(qA);
        z(17) = z(3)*z(5) - z(4)*z(6);
        z(2) = sin(qA);
        z(19) = z(3)*z(6) + z(4)*z(5);
        z(36) = z(1)*z(17) - z(2)*z(19);
        z(16) = z(5)*z(11) - z(6)*z(7);
        z(23) = L + z(16);
        z(18) = -z(3)*z(6) - z(4)*z(5);
        z(38) = z(1)*z(18) - z(2)*z(17);
        z(40) = z(13)*z(36) + z(23)*z(38);
        z(39) = z(1)*z(17) + z(2)*z(18);
        z(45) = L*z(39);
        z(42) = L*z(38);
        z(37) = z(1)*z(19) + z(2)*z(17);
        z(43) = z(13)*z(37) + z(23)*z(39);
        z(14) = W*z(6);
        z(15) = W*z(5);
        z(22) = L + z(15);
        z(41) = z(14)*z(36) + z(22)*z(38);
        z(44) = z(14)*z(37) + z(22)*z(39);
        z(46) = z(41)*z(45) - z(42)*z(44);
        z(47) = (z(40)*z(45)-z(42)*z(43))/z(46);
        qBp = -z(47)*qAp;
        z(48) = (z(40)*z(44)-z(41)*z(43))/z(46);
        qCp = z(48)*qAp;
        z(21) = z(12) + z(16);
        z(89) = IC + z(88)*z(21);
        z(20) = z(12) + z(15);
        z(93) = IC + z(88)*z(20);
        z(101) = z(89) + z(95)*z(48) - z(47)*z(93);
        z(9) = z(7) - HCOM;
        z(10) = 0.5*W + z(8);
        z(87) = z(86) + MC*(z(13)*z(14)+z(20)*z(21)) - 0.5*MB*(2*HCOM*z(9)-W*z(10));
        z(92) = z(91) + MC*(z(14)^2+z(20)^2);
        z(99) = z(87) + z(48)*z(93) - z(47)*z(92);
        z(108) = (z(41)*z(101)-z(42)*z(99))/z(46);
        z(26) = L*qAp;
        z(27) = qAp*z(26);
        z(28) = z(3)*z(27);
        z(29) = z(4)*z(27);
        z(30) = -0.5*W*(qAp+qBp)^2 - z(28);
        z(31) = z(29) - HCOM*(qAp+qBp)^2;
        z(32) = -W*(qAp+qBp)^2 - z(28);
        z(33) = z(5)*z(32) + z(6)*z(29);
        z(34) = z(5)*z(29) - z(6)*z(32);
        z(35) = z(33) - z(12)*(qAp+qBp+qCp)^2;
        z(49) = L*z(3)*qBp;
        z(50) = L*z(4)*qBp;
        z(51) = z(5)*z(11)*qCp + z(5)*z(49) - z(6)*z(7)*qCp - z(6)*z(50);
        z(52) = -z(3)*z(6)*qBp - z(3)*z(6)*qCp - z(4)*z(5)*qBp - z(4)*z(5)*qCp;
        z(53) = z(3)*z(5)*qBp + z(3)*z(5)*qCp - z(4)*z(6)*qBp - z(4)*z(6)*qCp;
        z(54) = z(1)*z(52) - z(1)*z(19)*qAp - z(2)*z(17)*qAp - z(2)*z(53);
        z(55) = -z(5)*z(7)*qCp - z(6)*z(11)*qCp - z(5)*z(50) - z(6)*z(49);
        z(56) = z(4)*z(6)*qBp + z(4)*z(6)*qCp - z(3)*z(5)*qBp - z(3)*z(5)*qCp;
        z(57) = z(1)*z(56) - z(1)*z(17)*qAp - z(2)*z(18)*qAp - z(2)*z(52);
        z(58) = z(13)*z(54) + z(23)*z(57) + z(36)*z(51) + z(38)*z(55);
        z(59) = W*z(5)*qCp;
        z(60) = W*z(6)*qCp;
        z(61) = z(14)*z(54) + z(22)*z(57) + z(36)*z(59) - z(38)*z(60);
        z(62) = qAp*z(58) + qBp*z(61) + L*qCp*z(57);
        z(63) = z(1)*z(17)*qAp + z(1)*z(53) + z(2)*z(52) - z(2)*z(19)*qAp;
        z(64) = z(1)*z(18)*qAp + z(1)*z(52) + z(2)*z(56) - z(2)*z(17)*qAp;
        z(65) = z(13)*z(63) + z(23)*z(64) + z(37)*z(51) + z(39)*z(55);
        z(66) = z(14)*z(63) + z(22)*z(64) + z(37)*z(59) - z(39)*z(60);
        z(67) = qAp*z(65) + qBp*z(66) + L*qCp*z(64);
        z(68) = (z(42)*z(67)-z(45)*z(62))/z(46);
        z(69) = (z(41)*z(67)-z(44)*z(62))/z(46);
        z(76) = z(1)*z(3) - z(2)*z(4);
        z(77) = z(1)*z(4) + z(2)*z(3);
        z(78) = -z(1)*z(4) - z(2)*z(3);
        z(85) = z(84) + MB*(z(9)^2+z(10)^2) + MC*(z(13)^2+z(21)^2);
        z(90) = MB*(z(9)*z(30)+z(10)*z(31)) + MC*(z(13)*z(35)+z(21)*z(34));
        z(94) = MC*(z(14)*z(35)+z(20)*z(34)) - 0.5*MB*(2*HCOM*z(30)-W*z(31));
        z(96) = z(88)*z(34);
        z(97) = z(85) + z(48)*z(89) - z(47)*z(87);
        z(98) = z(90) + z(87)*z(68) - z(89)*z(69);
        z(100) = z(94) + z(92)*z(68) - z(93)*z(69);
        z(102) = z(96) + z(93)*z(68) - z(95)*z(69);
        z(106) = (z(44)*z(101)-z(45)*z(99))/z(46);
        z(110) = z(43)*z(108) - z(97) - z(40)*z(106);
        z(118) = (z(41)*z(47)-z(40)-z(42)*z(48))*qAp;
        z(119) = (z(44)*z(47)-z(43)-z(45)*z(48))*qAp;
        
        % Specified perturbation
        if size(pert_mag,2)>1
            XPP = mdlPerturbation_usrInterp(T*INTEGSTP,pert_mag);
        else
            if length(pert_mag)>1
                XPP = mdlPerturbation_usrNoInterp(T,pert_mag);
            else
                if pert_mag<0
                    XPP = mdlPerturbation_step(T*INTEGSTP,-pert_mag);
                else
                    XPP = mdlPerturbation(T*INTEGSTP,pert_mag);
                end
            end
        end
        % Specified control
        if cntrl==1 %CoM
            TAB = mdlControl_com(T+1, [qB;qBp], 1, FF_GAINS, FB_GAINS, FB_DELAY);
            TBC =-mdlControl_com(T+1, [qC;qCp], 2, FF_GAINS, FB_GAINS, FB_DELAY);
        elseif cntrl == 2 %Hip
            TAB = mdlControl(T+1, [qB;qBp], 1, FF_GAINS, FB_GAINS, FB_DELAY);
            TBC =-mdlControl(T+1, [qC;qCp], 2, FF_GAINS, FB_GAINS, FB_DELAY);
        elseif cntrl==3 %One Hip only
            TAB = mdlControl(T+1, [qB;qBp], 1, FF_GAINS, FB_GAINS, FB_DELAY);
            TBC = 0;
        elseif cntrl==4 %CoM (only one torque applied)
            TAB = mdlControl_com(T+1, [qB;qBp], 1, FF_GAINS, FB_GAINS, FB_DELAY);
            TBC = 0;
        end
        
        z(74) = MC*XPP;
        z(83) = TBC - z(12)*(z(75)*z(39)-z(74)*z(38));
        z(105) = z(83) - z(102);
        z(72) = MB*XPP;
        z(82) = z(80)*z(77) + z(74)*z(14)*z(36) + z(74)*z(20)*z(38) + 0.5*W*z(72)*z(78) - TAB - 0.5*z(81)*z(76) - HCOM*z(72)*z(76) - z(75)*z(14)*z(37) - z(75)*z(20)*z(39);
        z(104) = z(82) - z(100);
        z(107) = (z(44)*z(105)-z(45)*z(104))/z(46);
        z(70) = MA*XPP;
        z(79) = z(72)*z(9)*z(76) + z(72)*z(10)*z(78) + z(74)*z(13)*z(36) + z(74)*z(21)*z(38) - z(73)*z(9)*z(77) - z(73)*z(10)*z(76) - z(75)*z(13)*z(37) - z(75)*z(21)*z(39) - LCOM*(z(71)*z(1)+z(70)*z(2));
        z(103) = z(79) - z(98);
        z(109) = (z(41)*z(105)-z(42)*z(104))/z(46);
        z(111) = z(103) + z(40)*z(107) - z(43)*z(109);
        z(120) = (z(47)*z(82)-z(79)-z(48)*z(83))*qAp;
        
        COEF(1,1) = z(110);
        RHS(1) = -z(111);
        SolutionToAxEqualsB(1) = RHS(1)/COEF(1,1);
        
        % Update variables after uncoupling equations
        qApp = SolutionToAxEqualsB(1);
        
        FC1 = z(107) - z(106)*qApp;
        FC2 = z(108)*qApp - z(109);
        WCHECK1p = z(120) + FC1*z(118) + FC2*z(119);
        
        % Update derivative array prior to integration step
        VARp(1) = qAp;
        VARp(2) = qBp;
        VARp(3) = qCp;
        VARp(4) = WCHECK1p;
        VARp(5) = qApp;
        
        sys = VARp';
        
        %%%% Evaluate output quantities
        if(u==1)
            Output = zeros(1,20);
            qBpp = z(68) - z(47)*qApp;
            qCpp = z(48)*qApp - z(69);
            COM_X = z(117) + z(113)*z(1) + z(115)*z(78) + 0.5*z(114)*z(76) - z(116)*z(36);
            COM_Y = z(113)*z(2) + z(115)*z(76) + 0.5*z(114)*z(77) - z(116)*z(37);
            COM_VX = 0.5*z(114)*z(78)*(qAp+qBp) - z(113)*z(2)*qAp - z(115)*z(76)*(qAp+qBp) - z(116)*z(38)*(qAp+qBp+qCp);
            COM_VY = z(113)*z(1)*qAp + 0.5*z(114)*z(76)*(qAp+qBp) - z(115)*z(77)*(qAp+qBp) - z(116)*z(39)*(qAp+qBp+qCp);
            z(121) = z(9) + HCOM*z(47);
            z(122) = z(10) - 0.5*W*z(47);
            z(123) = z(13) - z(14)*z(47);
            z(124) = z(21) + z(12)*z(48) - z(20)*z(47);
            z(126) = (z(125)+IB*(1-z(47))^2+MB*(z(121)^2+z(122)^2)+MC*(z(123)^2+z(124)^2)+IC*(-1+z(47)-z(48))^2)*qAp^2;
            ECheck = WCHECK1 + 0.5*z(126);
            % Kinematics
            Output(1)=T*INTEGSTP;  Output(2)=qA;  Output(3)=qB;  Output(4)=qC;
            Output(5)=qAp;  Output(6)=qBp;  Output(7)=qCp;
            Output(8)=qApp;  Output(9)=qBpp;  Output(10)=qCpp;
            % Constraint Force
            Output(11)=FC1;  Output(12)=FC2;
            % CoM trajectory
            Output(13)=COM_X;  Output(14)=COM_Y;
            Output(15)=COM_VX;  Output(16)=COM_VY;
            % Perturbation
            Output(17) = XPP;
            % Torque applied
            Output(18) = TAB; Output(19) = TBC;
            % Error checks (should all be zero)
            Output(20)=(L*z(1)+L*z(36)+W*z(76)-SW);
            Output(21)=(L*z(2)+L*z(37)+W*z(77));  Output(22)=ECheck;
            % Append answer to total solution
            aSolution(T+1,:) = Output;
        end
    end
%==========================================================================
% mdlOutput: Provides for a way to save output at every succesful step
%==========================================================================
    function status = mdlOutput(t)
        status = 0;
        if aSolution(t,12)<=0 %Foot comes off the ground
            status = 1;
        end
        if abs(aSolution(t,22))>1e-3 %Energy is growing very large
            status = 2;
        end
    end
%==========================================================================
% mdlPerturbation: Gives a Gaussian acceleration perturbation
%==========================================================================
    function xpp = mdlPerturbation(t,mag)
        % Parameters to create a 20 cm/s velocity perturbation
        % for 100 ms as is found in Jeven's thesis p31 would require
        % mag=0.4, t_stop=0.3
        % The current parameters give a 25 cm/s velocity perturbation for
        % 400 ms and result in a 12 cm displacement in the platform
        % mag=0.5, t_stop=0.6
        tc = t(end);
        t_start = 0.1;
        t_stop = 0.53;
        a_width = 0.0008;
        % A leftward acceleration is applied to the body, as if the
        % platform were accelerating to the right.  Since GRFs are only
        % calculated under the right leg, this direction is selected to
        % ensure detection of foot lift-off.
        xpp = mag*G*(-exp(-(tc-t_start)^2/a_width) ...  %acceleration
                     +exp(-(tc-t_stop)^2/a_width));     %deceleration
    end
%==========================================================================
% mdlPerturbation_step: Gives a Gaussian acceleration perturbation
%==========================================================================
    function xpp = mdlPerturbation_step(t,mag)
        % Function for creating a step perturbation for ranking stability
        % of feedback gains
        tc = t(end);
        t_start = 0.1;
        xpp = 0;
        % A leftward acceleration is applied to the body, as if the
        % platform were accelerating to the right.  Since GRFs are only
        % calculated under the right leg, this direction is selected to
        % ensure detection of foot lift-off.
        if tc>t_start
            xpp = -mag*G;
        end
    end
%==========================================================================
% mdlPerturbation: Gives a user supplied acceleration perturbation
%==========================================================================
    function xpp = mdlPerturbation_usrInterp(t,mag)
        % Get current time
        tc = t(end);
        % Interpolate for the current acceleration
        magc = interp1q(mag(:,1),mag(:,2),tc);
        
        xpp = magc*G;
    end
%==========================================================================
% mdlPerturbation: Gives a user supplied acceleration (no interpolation)
%==========================================================================
    function xpp = mdlPerturbation_usrNoInterp(t,mag)
        % Get current time
        tc = t(end)+1;
        xpp = mag(tc)*G;
    end
%==========================================================================
% mdlControl: Calculates a torque based on the state of a "hip"
%==========================================================================
    function aTorque = mdlControl(t, q, aHip, ff_gains, fb_gains, fb_delay)
        %Setup variables
        ffkp = ff_gains(1);        ffkv = ff_gains(2);
        fbkp = fb_gains(1);        fbkv = fb_gains(2);
        if aHip == 1
            qo = qBo;
            qi = 3;
        else
            qo = qCo;
            qi = 4;
        end
        
        %Feed-forward control
        aTorque = ffkp*(q(1)-qo) + ffkv*q(2);
        
        %Feed-back control
        if ~isempty(fb_delay) && ~isempty(aSolution) %Check if FBC possible
            if(t > fb_delay) %Check if delayed states exist
                %Determine delayed states
                qDelayed = aSolution(t-fb_delay,:);
                %Add on delayed feedback torque
                aTorque = aTorque + fbkp*(qDelayed(qi)-qo) ...
                                  + fbkv*qDelayed(qi+3);
            end
        end
    end
%==========================================================================
% mdlControl_com: Calculates a torque based on the state of the CoM
%==========================================================================
    function aTorque = mdlControl_com(t, q, aHip, ff_gains, fb_gains, fb_delay)
        %Setup variables
        ffkp = ff_gains(1);        ffkv = ff_gains(2);
        fbkp = fb_gains(1);        fbkv = fb_gains(2);
        if aHip == 1
            qo = qBo;
            qs = 1;
        else
            qo = qCo;
            qs = -1;
        end
        
        %Feed-forward control
        aTorque = ffkp*(q(1)-qo) + ffkv*q(2);
        
        %Feed-back control
        if ~isempty(fb_delay) && ~isempty(aSolution) %Check if FBC possible
            if(t > fb_delay) %Check if delayed states exist
                %Silence possible warning from interp when NANs are present
                warning('off', 'MATLAB:interp1:NaNinY');
                %Determine delayed states
                qDelayed = aSolution(t-fb_delay,:);
                %Add on delayed feedback torque
                aTorque = aTorque + qs*fbkp*(qDelayed(13)-SW/2) ...
                                  + qs*fbkv*qDelayed(15);
            end
        end
    end
end